import * as fs from 'fs/promises';
import { CustomError, ERROR_CODES, INDE_SQL_TYPES_MAP } from './various.js';
export class SqlGenerator {
    /**
     * Restituisce il nome fornito TUTTO MAIUSCOLO
     */
    get dbTableName() {
        if (!!this.dbName && this.dbName.length > 0 && this.params.compAsDbName) {
            return `${this.dbName}.${this.suppliedName}`;
        }
        else {
            return this.suppliedName;
        }
    }
    get fileName() {
        return `${this.suppliedName.toUpperCase()}.sql`;
    }
    constructor(name, config, dbName = '') {
        //proprietà della classe
        this.arProperties = [];
        //nome fornito
        this.suppliedName = '';
        //nome del db
        this.dbName = '';
        this.suppliedName = name;
        this.params = config;
        this.dbName = dbName;
    }
    /**
     * Aggiunge un campo alla tabella
     * @param name
     * @param type
     * @param required
     * @returns
     */
    addProperty(name, type, required = false, isPrimary = false) {
        if (!!name && name.length > 0 && !!type && type.length > 0 && required !== null && required !== undefined) {
            this.arProperties.push({
                name,
                type,
                required: required,
                isPrimary
            });
        }
        else {
            console.warn(`Error, property ${name} ignored, not all parameters supplied`);
        }
        return this;
    }
    /**
     * Aggiunge un campo chiave primaria alla tabella
     * @param name
     * @param type
     * @param required
     * @returns
     */
    addPrimaryKey(name, type) {
        this.addProperty(name, type, false, true);
        return this;
    }
    /**
     * Restituisce la stringa completa del file sql
     * @returns
     */
    getFileContentString() {
        const start = `CREATE TABLE ${this.params.ifNotExistCondition ? `IF NOT EXISTS` : ''} ${this.dbTableName} (\n`;
        const end = `\n)${this.generateRowId() ? '' : ' WITHOUT ROWID'}; \n\n`;
        const properties = this.getPropertiesString();
        const primaryKeys = this.getPrimaryKeysString();
        return (start +
            properties +
            primaryKeys +
            end);
    }
    generateRowId() {
        return this.params.generateRowId === null ? !this.existPrimaryKeys() : this.params.generateRowId;
    }
    existPrimaryKeys() {
        return this.arProperties.filter(el => el.isPrimary).length > 0;
    }
    getPropertiesString() {
        return this.arProperties
            .map(elProp => `\t${elProp.name} ${elProp.type}${elProp.required ? ' NOT NULL' : ''},\n`)
            .reduce((acc, el) => acc + el)
            .slice(0, -2);
    }
    getPrimaryKeysString() {
        const arKeys = this.arProperties.filter(el => el.isPrimary);
        const keysString = arKeys.length > 0 ?
            arKeys.map(el => el.name)
                .reduce((acc, el) => `${acc}, ${el}`)
            :
                '';
        return keysString.length > 0 ? `,\n\tPRIMARY KEY (${keysString})` : '';
    }
    /**
    * Salva la classe nel percorso specificato
    * @param path
    * @returns
    */
    saveOnFileSystem(path) {
        const fileContent = this.getFileContentString();
        const finalPath = `${path}${path.endsWith('/') ? '' : '/'}${this.fileName}`;
        return fs.writeFile(finalPath, fileContent)
            .catch(err => Promise.reject(new CustomError(err, ERROR_CODES.ERR_SAVING_MODEL_FILE)));
    }
    /**
     * converte il tipo proveniente da inde in tipo sql (se il tipo non è standard, verrà ricercato all'interno del array enumTypes passato)
     * @param indeType tipo proveniente da inde
     * @returns
     */
    static converToSqlType(indeType, enumTypes) {
        let finalType = null;
        let arTsTypes = Object.keys(INDE_SQL_TYPES_MAP);
        let isEnum = false;
        arTsTypes.forEach((key, idx, other) => {
            if (INDE_SQL_TYPES_MAP[key].includes(indeType))
                finalType = key;
        });
        if (finalType == null) {
            // il valore è di tipo enum, dobbiamo decodificarlo
            //cerchiamo l'enum giusto nella lista di enums che abbiamo
            const typeWithoutCmp = indeType.split('.')[1];
            const objEnum = enumTypes?.find(el => el.$.Name == typeWithoutCmp);
            if (!!objEnum) {
                //trovato! vediamo se l'enum dichiara il suo tipo
                if (!!objEnum.$.UnderlyingType) {
                    // l'enum dichiara il suo tipo, dobbiamo trasformarlo in un tipo sql
                    //un po' di ricorsione ci sta sempre bene
                    finalType = SqlGenerator.converToSqlType(objEnum.$.UnderlyingType);
                }
                else {
                    // l'enum non dichiara il suo tipo, dobbiamo provare ad evincerlo dai valori
                    const firstMember = objEnum.Member[0].$.Value;
                    if (this.isNumeric(firstMember)) {
                        finalType = 'REAL';
                    }
                    else {
                        finalType = 'TEXT';
                    }
                }
            }
            else {
                //non abbiamo trovato l'enum giusto, purtroppo non conosciamo il tipo
                finalType = 'UNKNOWN';
                isEnum = true;
            }
        }
        return finalType;
    }
    static isNumeric(value) {
        if (typeof value == 'number') {
            return true;
        }
        else if (typeof value == 'string') {
            //@ts-expect-error
            return !isNaN(value) && !isNaN(parseFloat(value));
        }
        else {
            return false;
        }
    }
}
//# sourceMappingURL=sql-generator.js.map